--Best = 167936

local M = {}

local helper = require('scenario/scenariohelper')
local file = require('scenario/fileHandler')
local powerupHelper = require('scenario/powerup')

local scenarioTimeOut = 0
local timeOutLimit = 5
local multiplier = 1
local startTimer = false
local reset = true
local powers = {false, false, false, false, false, false}
local dissolvedVehicles = {false, false}

local function success(score, team, opponent, multi)
	local medal, nextMedal
	if score < 4000 then 
		medal = 'None'
		nextMedal = 'Bronze(4000)'
	elseif score >= 4000 and score < 12000 then 
		medal = 'Bronze'
		nextMedal = 'Silver(12000)'
	elseif score >= 12000 and score < 24000 then 
		medal = 'Silver'
		nextMedal = 'Gold(24000)'
	elseif score >= 24000 and score < 40960 then 
		medal = 'Gold'
		nextMedal = 'Platinum(40960)'
	elseif score >= 40960 and score < 81920 then 
		medal = 'Platinum'
		nextMedal = 'Diamond(81920)'
	elseif score >= 81920 then 
		medal = 'Diamond'
		nextMedal = 'n/a'
	else
		medal = 'Broken'
		nextMedal = 'Also Broken'
	end

	file.setValueToSave(21, file.getMedalAsNumber(medal), 1)
	
    local result = {msg = 'Well Done! Your Score: (('..team..'-'..opponent..')*'..multi..') = '..score..' | You Achieved: '..medal..' | Next Medal: '..nextMedal..''}
	scenario_scenarios.finish(result)
end

local function onBeamNGTrigger(data)
	if data.triggerName == 'ice' and (data.subjectName == 'player' or data.subjectName == 'truck1' or data.subjectName == 'truck2') then
		helper.queueLuaCommandByName('player', 'bullettime.set(4)')
		if data.subjectName == 'player' then startTimer = true
		elseif powers[5] then
			local truck = scenetree.findObject('truck1')
			truck:queueLuaCommand('obj:setWind(0, 0, 0)')
		elseif powers[6] then
			local truck = scenetree.findObject('truck2')
			truck:queueLuaCommand('obj:setWind(0, 0, 0)')
		end
	end
end

local function checkPowerups(vehicleObject)
	if not powers[1] and powerupHelper.checkDistance(vehicleObject, 572.562988, 732.667053, 2.03013372, 3) then 
		powerupHelper.triggerMultiplier('x4', 0)
		powers[1] = true
		multiplier = multiplier*4
	end
	if not powers[2] and powerupHelper.checkDistance(vehicleObject, 570.398926, 724.484558, 6.0736165, 3) then
		powerupHelper.triggerMultiplier('x8', 1)
		powers[2] = true
		multiplier = multiplier*8
	end
	if not powers[3] and powerupHelper.checkDistance(vehicleObject, 543.738, 709.677, 2.55113, 3) then
		powerupHelper.triggerMultiplier('x8', 2)
		powers[3] = true
		multiplier = multiplier*8
	end
	if not powers[4] and powerupHelper.checkDistance(vehicleObject, 558.685974, 719.623474, 13.3396, 0.25) then
		powerupHelper.triggerMultiplier('x8', 3)
		powers[4] = true
		multiplier = multiplier*8
	end
	
	if not powers[5] and powerupHelper.checkDistance(vehicleObject, 561.724976, 747.991028, 1.94404995, 3) then
		powerupHelper.triggerMultiplier('juggernaut', 1)
		powers[5] = true
		--Find truck
		local truck = scenetree.findObject('truck1')
		--Apply acceleration to it
		truck:queueLuaCommand('obj:setWind(0, 300, 0)')
		timeOutLimit = timeOutLimit+5
	end
	if not powers[6] and powerupHelper.checkDistance(vehicleObject, 553.809814, 725.015, 1.94404721, 3) then
		powerupHelper.triggerMultiplier('juggernaut', 2)
		powers[6] = true
		--Find truck
		local truck = scenetree.findObject('truck2')
		--Apply acceleration to it
		truck:queueLuaCommand('obj:setWind(0, 300, 0)')
		timeOutLimit = timeOutLimit+5
	end
end

local function fail(reason)
  scenario_scenarios.finish({failed = reason})
end

local function checkAcid(vehicleObject, id)
	if vehicleObject:getPosition().y > -711 and vehicleObject:getPosition().z < 9.5 then
		vehicleObject:queueLuaCommand('fire.explodeVehicle()')
		vehicleObject:queueLuaCommand('beamstate.breakAllBreakgroups()')
		if id == 'player' then fail('Your car has been dissolved!')
		else
			dissolvedVehicles[id] = true
			helper.flashUiMessage('Opponent has been dissolved!', 3)
		end
	end
end

local function onPreRender(dt)
	if reset then
		TorqueScript.eval([[
			x4_board.position = "-572.562988 -732.667053 2.03013372";
			x8_1_board.position = "-570.398926 -724.484558 6.0736165";
			x8_2_board.position = "-543.738 -709.677 2.55113";
			x8_3_board.position = "-558.685974 -719.623474  13.3396";
			juggernaut_1_board.position = "-561.724976 -747.991028 1.94404995";
			juggernaut_2_board.position = "-553.809814 -725.015 1.94404721";
		]])
		reset = false
	end
	
	checkPowerups(scenetree.findObject('player'))
	checkPowerups(scenetree.findObject('opponent1'))
	checkPowerups(scenetree.findObject('opponent2'))
	checkPowerups(scenetree.findObject('truck1'))
	checkPowerups(scenetree.findObject('truck2'))
	
	checkAcid(scenetree.findObject('player'), 'player')
	if not dissolvedVehicles[1] then checkAcid(scenetree.findObject('opponent1'), 1) end
	if not dissolvedVehicles[2] then checkAcid(scenetree.findObject('opponent2'), 2) end
end

local function finish()
	local playerObject = scenetree.findObject('player')
    local carOpponentObject1 = scenetree.findObject('opponent1')	
	local carOpponentObject2 = scenetree.findObject('opponent2')		
	local distance = math.floor(math.sqrt(math.pow(playerObject:getPosition().x+563.3, 2) + math.pow(playerObject:getPosition().y+733.6, 2), math.pow(playerObject:getPosition().z+0.01, 2)))
	local distanceOpponent1 = math.floor(math.sqrt(math.pow(carOpponentObject1:getPosition().x+563.3, 2) + math.pow(carOpponentObject1:getPosition().y+733.6, 2), math.pow(carOpponentObject1:getPosition().z+0.01, 2)))
	local distanceOpponent2 = math.floor(math.sqrt(math.pow(carOpponentObject2:getPosition().x+563.3, 2) + math.pow(carOpponentObject2:getPosition().y+733.6, 2), math.pow(carOpponentObject2:getPosition().z+0.01, 2)))
	
	local scorePlayer =    math.max(0, 100-distance*2)
	local scoreOpponent1 = math.max(0, 100-distanceOpponent1*2)
	local scoreOpponent2 = math.max(0, 100-distanceOpponent2*2)
	
	if dissolvedVehicles[1] then scoreOpponent1 = 0 end
	if dissolvedVehicles[2] then scoreOpponent2 = 0 end

	local scoreFinal = (scorePlayer-scoreOpponent1-scoreOpponent2)*multiplier
	success(scoreFinal, scorePlayer, scoreOpponent1+scoreOpponent2, multiplier)
end

local function onRaceTick(raceTickTime)
	local playerObject = scenetree.findObject('player')
	if startTimer then
		scenarioTimeOut = scenarioTimeOut+raceTickTime
		playerObject:queueLuaCommand('obj:setWind(0, 0, 0)')
	else 
		playerObject:queueLuaCommand('obj:setWind(0, 200, 0)')
	end
	
	if scenarioTimeOut == 5 then helper.queueLuaCommandByName('player', 'bullettime.set(1)') end
	
	if scenarioTimeOut >= timeOutLimit then 
		if (not powers[5] and not powers[6]) or timeOutLimit == 15 then finish() end
	end
end

M.onRaceTick = onRaceTick
M.onBeamNGTrigger = onBeamNGTrigger
M.onPreRender = onPreRender
return M