local M = {}

--Number of medals/trophies that can be won
local currentMedals = 20
local currentTrophies = 4

local function reformatFile()
	local file = io.open("cargames_save.txt", "r")
	local line = file:read()
	local firstLine, secondLine = {}, {}
	
	for i = 1, #line do
		local digit = tonumber(line:sub(i, i))
		if i == 1 then --Darts Trophy
			if digit == 6 then
				table.insert(secondLine, 1) --If achieved then put in 1 to file
				table.insert(firstLine, 5) --And put diamond in first line
			else
				table.insert(secondLine, 0) --Else put 0 in file
				table.insert(firstLine, digit) --And put medal in first line
			end
		elseif i == 15 then --Singularity Trophy
			if digit == 6 then
				table.insert(secondLine, 1)
				table.insert(firstLine, 5)
			else 
				table.insert(secondLine, 0) --Else put 0 in file
				table.insert(firstLine, digit)
			end
		elseif i >= 16 and i<= 19 then table.insert(secondLine, digit) --Trophies go into second line
		else table.insert(firstLine, digit) --Medals stay on the first line
		end
	end
	
	local line1, line2 = '', '' --Remake the 'number' as a string
	for key, value in pairs(firstLine) do --Loops through table of medals
		local subString = value
		line1 = line1 .. subString --Adds resultant string with next string in table
	end
	
	secondLine[1], secondLine[2] = secondLine[2], secondLine[1] --Switch Darts and Singularity around
	for key, value in pairs(secondLine) do
		local subString = value
		line2 = line2 .. subString
	end
	
	local file = io.open("cargames_save.txt", "w")
	file:write(line1, '\n')
	file:write(line2, '\n')
	file:write('000000000') --Portal Gems (Green)
	file:close()
end

local function shouldReformat() --For updating file
	local file = io.open("cargames_save.txt", "r")
	local numLines = 0
	for line in file:lines() do numLines = numLines+1 end
	
	if numLines == 1 then return true
	elseif numLines >= 2 then return false
	else return nil
	end
end

local function checkFileExists()
	local file = io.open("cargames_save.txt", "r")
	if file == nil then
		file = io.open("cargames_save.txt", "w")
		file:write('00000000000000000000', '\n') --Medals
		file:write('000000', '\n') --Trophies
		file:write('000000000') --Portal Gems (Green)
		file:close()
	else
		if shouldReformat() then reformatFile() end
		file:close()
	end
end

--Used to update the savefile
--                  medalAchieved, indexOfScenario, indexOfTrophy, numberOfTrophy

local function updateFile(newMedal, scenarioDigit, trophyDigit, trophy)
	checkFileExists()
	local file = io.open("cargames_save.txt", "r")
	
	local medals = file:read()
	file:close()
	
	--For updating - adds in required digits to savefile to make save successful
	if #medals < currentMedals then 
		local digitsToAdd = currentMedals-#medals --Number of digits needed to add in
		local newString = medals --Current string

		for i = 1, digitsToAdd do --Loops through table of medals
			newString = newString .. '0' --Adds resultant string with 0 as default
		end
		
		local file = io.open("cargames_save.txt", "w")
		file:write(newString)
		file:close()
		
		medals = newString
	end
	
	local medalsTable = {}
	local update, forceUpdate = true
	--loop through save 'number'
	for i = 1, #medals do
		local number = tonumber(medals:sub(i, i))		
		if trophyDigit ~= nil and i == trophyDigit then 
			number = trophy
			forceUpdate = true
		elseif i == scenarioDigit then
			--Update medal if previus medal is less
			if number < newMedal then number = newMedal 
			else update = false
			end
		end
		table.insert(medalsTable, number)
	end
	
	if update or forceUpdate then --Whether a medal needs updating
		local newString = '' --Remake the 'number' as a string
		for key, value in pairs(medalsTable) do --Loops through table of medals
			local subString = value
			newString = newString .. subString --Adds resultant string with next string in table
		end
		
		local file = io.open("cargames_save.txt", "w")
		file:write(newString)
		file:close()
	end
end

local function getLineFromSave(lineNum)
	checkFileExists()
	local file = io.open("cargames_save.txt", "r")
	local i = 1
	for line in file:lines() do
		if i == lineNum then 
			file:close()
			return line 
		end
		i = i+1
	end
	file:close()
end

local function getListOfMedals()
	checkFileExists()
	local file = io.open("cargames_save.txt", "r")
	
	local fileList = getLineFromSave(1)
	file:close()
	
	local bronze, silver, gold, platinum, diamond = 0, 0, 0, 0, 0
	for i = 1, #fileList do
		local medal = tonumber(fileList:sub(i, i))
		if medal == 1 then     bronze   = bronze  +1
		elseif medal == 2 then silver   = silver  +1
		elseif medal == 3 then gold     = gold    +1
		elseif medal == 4 then platinum = platinum+1
		elseif medal == 5 then diamond  = diamond +1
		end
	end
	
	return bronze, silver, gold, platinum, diamond
end

local function getValueFromSave(digit, line)
	local fileList = getLineFromSave(line)
	return tonumber(fileList:sub(digit, digit))
end

local function getTableLength(t)
	local length = 0
	for _ in pairs(t) do length = length+1 end
	return length
end

local function setLineToSave(data, lineNum)
	checkFileExists()
	local f = io.open("cargames_save.txt", "r")
	local i = 1
	local linesToAdd = {}
	for line in f:lines() do
		if i == lineNum then line = data end
		table.insert(linesToAdd, line)
		i = i+1
	end
	f:close()
	
	local tableLength = getTableLength(linesToAdd)
	
	local file = io.open("cargames_save.txt", "w")
	local newString = '' --Remake the 'number' as a string
	for key, value in pairs(linesToAdd) do 
		if key == tableLength then --Last element in table
			file:write(value) --Same as below but no new line afterwards to prevent issues with blankspaces
		else file:write(value, '\n') 
		end
	end
	file:close()
end

local function setValueToSave(index, value, line)
	checkFileExists()

	local i = 1
	local saveLine = {}
	local fileList = getLineFromSave(line)
	
	for j = 1, #fileList do --Loop through file line
		local digitValue = tonumber(fileList:sub(j, j))
		if j == index then
			if digitValue < value then digitValue = value end --Sets the value to the digit for saving to table
		end
		table.insert(saveLine, digitValue)
	end
	
	local newString = '' --Remake the 'number' as a string
	for key, value in pairs(saveLine) do --Loops through table of medals
		local subString = value
		newString = newString .. subString --Adds resultant string with next string in table
	end
	
	setLineToSave(newString, line)
end

local function getMedalAsNumber(medal)
	if     medal == 'None'    							 then return 0
	elseif medal == 'Bronze'  							 then return 1
	elseif medal == 'Silver'  							 then return 2
	elseif medal == 'Gold'     							 then return 3
	elseif medal == 'Platinum' 							 then return 4
	elseif medal == 'Diamond' or medal == 'Singularity'  then return 5
	end
end

M.updateFile       = updateFile
M.getMedalAsNumber = getMedalAsNumber
M.checkFileExists  = checkFileExists
M.getValueFromSave = getValueFromSave
M.setValueToSave   = setValueToSave
M.getListOfMedals  = getListOfMedals
M.getLineFromSave  = getLineFromSave
M.setLineToSave    = setLineToSave
return M