local M = {}
local fileHandling = require('scenario/fileHandler')
local portal = {["scaleX"] = 1, ["scaleZ"] = 0.75}
local stabilisers = {["rot"] = 0}
local control = {["energyStored"] = 0, ["crystals"] = 0, ["catStored"] = 0, ["catReq"] = 100}
local pState = 'off'

local function fuel()
	local fuelLine = fileHandling.getLineFromSave(3)
	local fuel = 0
	for i = 1, #fuelLine do
		if tonumber(fuelLine:sub(i, i)) == 1 then fuel = fuel+1 end
	end
	
	control["crystals"] = fuel
	ui_message('Fuel Level: '..fuel..' /9', 10, 'A')
end

local function applyCatalysts()
	local b, s, g, p, d = fileHandling.getListOfMedals()
	--Bronze   1x
	--Silver   2x
	--Gold     4x
	--Platinum 6x
	--Diamond  10x
	control["catStored"] = b*1 + s*2 + g*4 + p*6 + d*10
	ui_message('Catalysts Stored: '..control["catStored"]..' /100', 10, 'B')
end

---------------------------------------------------------------------------------------------
--Particles--
---------------------------------------------------------------------------------------------

local function getRelativeCoords(radius1, radius2) --Basically circular motion
	--in the form (rcosθ, rsinθ)
	local x = radius1*math.cos(radius1*5)
	local y = radius2*math.sin(radius1*5)
	return x, y
end

local function portalParticles(radius1, radius2)
	local x, y = getRelativeCoords(radius1, radius2)
	TorqueScript.eval('portalParticle1.position = "'..(-933.142761+x)..' -1003.56543 '..(125.14534+y)..'";')
end


---------------------------------------------------------------------------------------------
--Main Control--
---------------------------------------------------------------------------------------------

local function animate()
	stabilisers["rot"] = stabilisers["rot"]+control["energyStored"]
	TorqueScript.eval([[
		test1.rotation = "-1 0 0 ]]..(stabilisers["rot"]*1  )..[[";
		test2.rotation = "-1 0 0 ]]..(stabilisers["rot"]*1.1)..[[";
		test3.rotation = "-1 0 0 ]]..(stabilisers["rot"]*1.2)..[[";
		test4.rotation = "-1 0 0 ]]..(stabilisers["rot"]*1.2)..[[";
		test5.rotation = "-1 0 0 ]]..(stabilisers["rot"]*1.1)..[[";
		test6.rotation = "-1 0 0 ]]..(stabilisers["rot"]*1  )..[[";
		bwPortal.scale = "]]..portal["scaleX"]..[[ 0 ]]..portal["scaleZ"]..[[";
	]])
end

local function adjustToroids(increment)
	control["energyStored"] = control["energyStored"]+increment
	if control["energyStored"] < 100 and control["energyStored"] > 0 then
		ui_message('Speed: '..math.floor(control["energyStored"])..'%', 1, 'A')
		stabilisers["rot"] = stabilisers["rot"]+control["energyStored"]
	else
		if control["energyStored"] >= 100 then 
			control["energyStored"] = 100 --Force it to have no extra charge incase of framrate incosistencies
			pState = 'powered'
		elseif control["energyStored"] <= 0 then 
			control["energyStored"] = 0 --Force it to have no residual charge incase of framrate incosistencies
			ui_message('Portal Successfully Closed!', 10, 'A')
			pState = 'off'
		end
	end
end

local function adjustPortal(i1, i2)
	portal["scaleX"] = portal["scaleX"]+i1
	portal["scaleZ"] = portal["scaleZ"]+i2
	if portal["scaleX"] < 70 and portal["scaleX"] > 0 then ui_message('Stabilisation: '..math.floor(portal["scaleX"]/70*100)..'%', 1, 'A')
	else
		if portal["scaleX"] >= 70 then 
			pState = 'on'
			ui_message('Portal Open!', 10, 'A')
		elseif portal["scaleX"] <= 0 then 
			portal["scaleX"] = 0 --Force no scale
			portal["scaleZ"] = 0 --Force no scale
		end
	end
	
	portalParticles(portal["scaleX"], portal["scaleZ"])
end

local function update() --Should always be running
	if pState == 'energise' then adjustToroids(0.1)
	elseif pState == 'enervate' then 
		adjustToroids(-0.1)
		adjustPortal(-0.2, -0.15)
	elseif pState == 'powered' then adjustPortal(0.04, 0.03)
	end
	
	animate()
end

local function setupPortal()
	fuel() --Update fuel level
	applyCatalysts() --Update catalysts

	local catalysts, fuel = false, false
	if control["catStored"] >= control["catReq"] then catalysts = true
	else ui_message('Not enough catalysts!', 10, 'b')
	end
	
	if control["crystals"] == 9 then fuel = true
	else ui_message('Not enough crystals!', 10, 'c')
	end
	
	if catalysts and fuel then 
		control["energyStored"] = 0.01
		pState = 'energise' 
	end
end

local function closePortal() pState = 'enervate' end

local function handleInput(data)
	if data.triggerName == 'p_stop' and pState == 'on' then closePortal()
	elseif data.triggerName == 'p_start' and pState == 'off' then setupPortal()
	end
end

M.update = update
M.handleInput = handleInput
M.closePortal = closePortal
M.setupPortal = setupPortal
M.animate = animate
return M